#! /usr/bin/perl

## "@(#) $Id: ferosMJD2DSofN.pl,v 1.4 2005/11/20 06:22:06 jpritcha Exp $"
##
## This version is taken directly from:
## @(#) Id: ferosMJD2DSofN.plx,v 0.237 2005/11/12 16:00:41 vltsccm Exp

## For this to work the Astro::Time module must be installed into...
##    /vlt/INTROOT/FEROS/perl/Astro
## But I haven't yet worked out how to make the VLT makefile do this
## automatically
## Also should use env variable PERLLIB or PERL5LIB rather than hardwiring
## the lib path here.
use lib "/vlt/INTROOT/FEROS/lib/perl";
use Astro::Time;
use Carp;
use Getopt::Std;

%options=();
getopts("dfm",\%options);

##############################################################################
if ( $options{d} ) { $debug = 1; } # Used for debugging
##############################################################################

my $numarg=$#ARGV;
my $prog=filePart($0);
my $str=shift;
die "Usage: $prog [-f] <mjd|YYYY-MM-DDThh:mm:ss.sss|YYYY-MM-DDTsssss.s>

  -d debug
  -f write to stdout the full VLT-compliant date string
  -m write to stdout the MJD-OBS instead of the date string
  
" unless defined $str ;

## The TZ correction forces the change of date at LOCAL midday according to
## the TZ setting in the shell running the script.

# The following section hacked from Astro::Time (Time.pm, version 1.10)
# obtained from
# http://search.cpan.org/~cphil/Astro-0.62/Astro/Time.pm
# Time.pm is archived in this CMM module as AstroTime.pm
  # Does it match YYYY-MM-DDTdd:dd:dd.d form
  if ($str =~ /^\s*(?:([+-])\s*)?      # Sign (optional)
	         (\d*)-                # Year
	         (\d*)-                # Month
	         (\d*)T                # Day
	         (\d*):                # Hours degrees
	         (\d{0,2})(?::         # Minutes
	         (\d{0,2}(?:\.\d*)?))? # Seconds and fractions (both optional)
                /x) {
    print STDERR "Matches YYYY-MM-DDThh:mm:ss.s\n" if $debug;
    my $sign = -1 if (defined($1) && $1 eq "-");
    my $year = $2 if ($2);
    my $month = $3 if ($3);
    my $day = $4 if ($4);
    my $hour = $5 if ($5);
    my $min = $6 if ($6);
    my $sec = $7 if ($7);
    printf(STDERR "Year = %d, Month = %d, Day = %d, hour = %d, min = %d, sec = %f\n",$year,$month,$day,$hour,$min,$sec)  if $debug;;
    $mjd = cal2mjd($day,$month,$year,hms2time($hour,$min,$sec,H));
    printf(STDERR "MJD = %14.8f\n",$mjd) if $debug;;
  } elsif ($str =~ /^\s*(?:([+-])\s*)?      # Sign (optional)
                 (\d*)-                # Year
                 (\d*)-                # Month
                 (\d*)T                # Day
                 (\d{0,5}(?:\.\d*)?)? # Seconds since 00:00 and fractions (both optional)
                /x) {
    print STDERR "Matches YYYY-MM-DDTsssss.s\n" if $debug;
    my $sign = -1 if (defined($1) && $1 eq "-");
    my $year = $2 if ($2);
    my $month = $3 if ($3);
    my $day = $4 if ($4);
    my $tsec = $5 if ($5);
    printf(STDERR "Year = %d, Month = %d, Day = %d, sec = %f\n",$year,$month,$day,$tsec)  if $debug;;
    $mjd = cal2mjd($day,$month,$year,($tsec/24.0/60.0/60.0));
    printf(STDERR "MJD = %14.8f\n",$mjd) if $debug;;
  } elsif ($str =~ /^\s*(?:([+-])\s*)?(\d+(?:\.\d*)?)/) {
    print STDERR "Matches dd.d\n" if $debug;
    $sign = -1 if (defined($1) && $1 eq "-");
    $mjd = $2 if ($2);
    my($day, $month, $year, $ut) = mjd2cal($mjd);
    my($sign,$hour,$min,$sec) = time2hms($ut,H,6);
    printf( STDERR "%04d-%02d-%02dT%02d:%02d:%06.3f\n",$year,$month,$day,$hour,$min,$sec) if $debug;
  } else {
    exit 1;
  }

  if ( $options{m} ) {
    printf("%14.8f\n",$mjd);
  } else {
    if ( $options{f} ) {
      my($day, $month, $year, $ut) = mjd2cal($mjd);
      my($sign,$hour,$min,$sec) = time2hms($ut,H,6);
      printf( "%04d-%02d-%02dT%02d:%02d:%06.3f\n",$year,$month,$day,$hour,$min,$sec);
    } else {
      my($day, $month, $year, $ut) = mjd2cal($mjd - 0.5 - TZOffset()/24.0/3600.0 );
      printf("%4.4d-%2.2d-%2.2d\n",$year,$month,$day);
    }
  }
# Calculate the timezone offset in seconds.
sub TZOffset {
  use Time::gmtime;
  use Time::localtime;
  use Time::Local;
  local($gm)=gmtime();
  local($lt)=localtime();
  local($tzoff)=timelocal($gm->sec(),$gm->min(),$gm->hour(),$gm->mday(),$gm->mon(),$gm->year())-timelocal($lt->sec(),$lt->min(),$lt->hour(),$lt->mday(),$lt->mon(),$lt->year());
  return ( sprintf("%06d",$tzoff) );
}

## filePart taken from mkRHCD by A.J.Richards.
sub filePart {
	local($path)=@_;
	local($len)=length($path);

	for($i=$len;$i>0;$i--) {
		if (substr($path,$i,1) eq "/") {
			return(substr($path,$i+1));
		}
	}
	return($path);
}
