#! /usr/bin/perl

## "@(#) $Id: ferosMkHTML.pl,v 1.22 2006/02/02 11:46:25 jpritcha Exp $"

## For this to work the Astro::Time module must be installed into...
##    /vlt/INTROOT/FEROS/perl/Astro
## But I haven't yet worked out how to make the VLT makefile do this
## automatically
## Also should use env variable PERLLIB or PERL5LIB rather than hardwiring
## the lib path here.

####Hereafter,  v 1.2 by Xavier Haubois-ESO/la Silla- August 2004 :

####This humble script creates in an automatically way webpages containing general (for every kind of type) then detailled informations (for reduced science type only ) about files coming from observations, calibrations or maintenance with FEROS. This script should be inserted into the reduction pipeline of FEROS. This should allow the astronomer to have all the details about calibrations and science exposures he might require in webpages. The goal is to get all the relevant informations as soon as possible and look at them in a convenient way to improve the control of the operations and thus the efficiency of the observations.  

##Syntax :  ferosMkHTML.pl [options] file
### [options] : In the order of specification,
############## -d : debug 
############## -a add : Just write/add the file informations in the Master Webpage ; for every type of file.  
############## -a mklink : Only for the SCIENCE category (types : Sky or Calibration), add the file in the Master Webpage creating a link to the index webpage specific to the file. This page contains plots and details. If the file has been previously added, it just makes the link and the index webpage.
### file : Should be like that in the following format : feroNNNN.mt 

use lib "/vlt/INTROOT/FEROS/lib/perl";
use Astro::Time;
use Carp;
use Getopt::Std;
use File::Copy;
use File::Basename;
use File::Path;

my %options=();
getopts("a:drt:F:G:W:",\%options);


##############################################################################
if ( $options{d} ) { $debug = 1; } # Used for debugging
my $bgamp="&" unless ( $options{d} );
my $sysexec="exec";
$sysxec="system" unless ( $options{d} );
$sysexec="system";
##############################################################################

print("\n") if $options{d};
my $fileinfo;
my $HeadingLabel;
my $SectionLabel;
my $numarg=$#ARGV;
my $prog=basename($0);

### Usage...
my $ustr="
Usage: $prog [-d] <-a <ACTION>> [-t <EXPTYPE>] <file>
       $prog [-dr] <-a <ACTION>> -t DRSINIT -F <Flat.cat> -W <WLC.cat> -G <Guess> <file>

Note: All options MUST be given BEFORE the <file> argument.
  -a <ACTION>: One of 'add', 'mklink' or 'display'
  -d: debug
  -r: Display Reference DRS initialisation (only used if ACTION = display)
  -t <EXPTYPE>: Exposure type as defined in autoreduce.prg, i.e. one of
BIAS, DARK, FLATFIELD, CALIBRATION or SCIENCE, or is DRSINIT. If -t is NOT
provided, EXPTYPE is read from the header. EXPTYPE MUST be given for DRSINIT.
If EXPTYPE=DRSINIT the <file> should be the first WLC image (e.g. fero0111.mt)
and the following relevant info must be given:
  -F <Flat.cat>: flat field image catalogue, e.g. FF.cat
  -W <WLC.cat>: Wavelength calibration image, e.g. ThAr.cat
  -G <Guess>: guess session used, e.g. ThAr0196
";

my $file=shift;
die "${ustr}" unless defined $file ;
die "\nFILE $file DOESN'T EXIST\n\n" unless -e $file ;
$fn=basename($file);
$dir=dirname($file);

chomp($options{t}=`gethead $file 'HIERARCH ESO DPR TYPE'`.`gethead $file 'EXPTYPE'`) unless defined $options{t};
print("EXPTYPE = $options{t}\n") if $options{d} ;
die "${ustr}" unless defined $options{a} ;
if ( $options{t} =~ /DRSINIT/ ) {
  die "${ustr}" unless defined $options{F} ;
  die "${ustr}" unless defined $options{G} ;
  die "${ustr}" unless defined $options{W} ;
  die "\nFILE $options{F} DOESN'T EXIST\n\n" unless -e $options{F} ;
  die "\nFILE $options{W} DOESN'T EXIST\n\n" unless -e $options{W} ;
#  die "\nFILE $options{G} DOESN'T EXIST\n\n" unless -e $options{G} ;
}

# Use gethead to read value of MJD-OBS descriptor from file.
#    gethead <file> <keyword>
## is this a FIERA generated image???
## Old BIAS frames don't have MJD-OBS, but they do have DATE-OBS & TM_START
my $isFIERA=`gethead $file 'HIERARCH ESO DPR CATG'`;
printf("isFIERA = $isFIERA\n") if $options{d} ;;
my $mjd="a";
if ( $isFIERA =~ /^$/ ) {
  chomp($mjd=`gethead $file DATE-OBS`);
  chomp($mjd="${mjd}T".`gethead $file TM_START`);
} else {
## Use Time of OB start if available...
  chomp($mjd=`gethead $file 'HIERARCH ESO OBS START'`);
  if ( $mjd =~ /^$/ ) { chomp($mjd=`gethead $file MJD-OBS`); }
}
printf("MJD = $mjd\n") if $options{d} ;;

# Use ferosMJD2DSofN.pl to calculate the "Date of Start of Night"
# to be used in html file name.
#    ferosMJD2DSofN.pl <mjd|YYYY-MM-DDThh:mm:ss.sss|YYYY-MM-DDTsssss.s>
chomp(my $DSoN=`ferosMJD2DSofN.pl $mjd`);
print("$DSoN\n") if $options{d} ;
# HTML files should be created in directory $FEROS_DRS/html and
# subdirectories thereof.

### Add date into Summary page if needed...
my $ofile="$ENV{FEROS_DRS}/html/FEROS-DRS-Products.html";
if ( ! -e $ofile ) {
  copy ("$ENV{FEROS_DRS}/html/FEROS-Template-DRS-Products.html",$ofile) or die "Can't create webpage $ofile";
}
open( FILE , "$ofile" );
@ifile = <FILE>;
close(FILE);
grep( /$DSoN/ , @ifile );
my $daif = grep( /$DSoN/ , @ifile );
if ( $daif == 0 ) {
  open( FILE , "> $ofile" );
  foreach (@ifile) {
    s,<!-- <li> Add next Date here</li> -->,  <li><a href="FEROS-DRS-${DSoN}.html">${DSoN}</a><br>\n<!-- <li> Add next Date here</li> -->,g ;
    print FILE ;
  }
  close(FILE);
}

###Create the Master Webpage(corresponding to one observation night) 
my $ofile="$ENV{FEROS_DRS}/html/FEROS-DRS-${DSoN}.html";
if ( ! -e $ofile ) {
  copy ("$ENV{FEROS_DRS}/html/FEROS-Template-DRS.html",$ofile) or die "Can't create webpage $ofile";
}

###Fetch relevant informations from the header
my $fileinfo = mkFileInfo( $options{a} );
my $FFroot;  my @FFfiles;  my $FFfim;  my $FFim;  my $FFfimArcName;  my $FFfimOrigname;
my $WLCroot; my @WLCfiles; my $WLCfim; my $WLCim; my $WLCfimArcName; my $WLCfimOrigname;
if ( $options{t} =~ /DRSINIT/ ) {
  $FFroot=basename($options{F},".cat");
  open(FFcat,$options{F});
  @FFfiles = grep {/fero/} <FFcat>;
  $FFfim = substr($FFfiles[0],0,11);
  $FFim = $FFroot.substr($FFfiles[0],4,4);
  chomp($FFfimArcName=`gethead $FFfim ARCFILE`);
  chomp($FFfimOrigname=`gethead $FFfim ORIGFILE`);

  $WLCroot=basename($options{W},".cat");
  open(WLCcat,$options{W});
  @WLCfiles = grep {/fero/} <WLCcat>;
  $WLCfim = substr($WLCfiles[0],0,11);
  $WLCim = $WLCroot.substr($WLCfiles[0],4,4);
  chomp($WLCfimArcName=`gethead $WLCfim ARCFILE`);
  chomp($WLCfimOrigname=`gethead $WLCfim ORIGFILE`);

#  $arcfilebis="DRSINIT-".$FFim."-".$WLCim."-".$options{G};
#  print("ArcfileBis = $arcfilebis\n") if $options{d};
  $arcfile="DRSINIT-".substr($FFfimArcName,6,23)."_".substr($WLCfimArcName,6,23)."_".$options{G};
  $arcfilebis=$arcfile;
  print("ArcfileBis = $arcfilebis\n") if $options{d};
}
###Store these particurlarly relevant informations into the logInfo string
my $logInfo = mkLogInfo();

###Check if right target diretories exist. If not it creates them ! 
if ( ! -d "$ENV{FEROS_DRS}/html" ) { mkdir("$ENV{FEROS_DRS}/html") or die "Can't create directory $ENV{FEROS_DRS}/html" ; }
if ( ! -d "$ENV{FEROS_DRS}/html/${DSoN}" ) { mkdir("$ENV{FEROS_DRS}/html/${DSoN}") or die "Can't create directory $ENV{FEROS_DRS}/html/${DSoN}" ; }
if ( ! -d "$ENV{FEROS_DRS}/html/${DSoN}/${arcfilebis}" ) { mkdir("$ENV{FEROS_DRS}/html/${DSoN}/${arcfilebis}") or die "Can't create directory $ENV{FEROS_DRS}/html/${DSoN}/${arcfilebis}" ; }
open( FILE , "$ofile" );
@ifile = <FILE>;
close(FILE);

###Sort according to the type and prepare informations such as the different names and logInfo to be written onto the Master Webpage
print("EXPTYPE = $options{t}\n") if $options{d} ;
SWITCH: for ( $options{t} ) {

  /BIAS/    && do {
    print("This is a BIAS image\n") if $options{d} ;
    $HeadingLabel="<h3>CALIBRATION exposures</h3>";
    $SectionLabel="<h4>BIAS</h4>";
      SWITCH: for ( $options{a} ) {

        /mklink/    && do {
          $fileinfo="<tt><a href=\"${DSoN}/${arcfilebis}/index.html\">$arcfile</a> == $origname == ${fn}</tt><br>";
          last; 
        };

        /add/    && do {
          $fileinfo="<tt>$arcfile == $origname == ${fn}</tt><br>";
          last;
        };
      }
   last;
  };


  /DARK/    && do {
    print("This is a DARK image\n") if $options{d} ;                  
    $HeadingLabel="<h3>CALIBRATION exposures</h3>";
    $SectionLabel="<h4>DARK</h4>";
      SWITCH: for ( $options{a} ) {

        /mklink/    && do {
          $fileinfo="<tt><a href=\"${DSoN}/${arcfilebis}/index.html\">$arcfile</a> == $origname == ${fn}</tt><br>";
          last; 
        };

        /add/    && do {
          $fileinfo="<tt>$arcfile == $origname == ${fn}</tt><br>";
          last;
        };
      }
   last; 
  };


  /FLAT|FLATFIELD/    && do {
    if ( `gethead $file 'HIERARCH ESO TPL ID'` =~ /FEROS_ech_cal_df/ ) {
      print("This is a DOME FLATFIELD image\n") if $options{d};
      $HeadingLabel="<h3>CALIBRATION exposures</h3>";
      $SectionLabel="<h4>Dome FlatFields</h4>";
      $options{t}="DOMEFLAT";
      SWITCH: for ( $options{a} ) {

        /mklink/    && do {
          $fileinfo="<tt><a href=\"${DSoN}/${arcfilebis}/index.html\">$arcfile</a> == $origname == ${fn}</tt><br>";
          last; 
        };

        /add/    && do {
          $fileinfo="<tt>$arcfile == $origname == ${fn}</tt><br>";
          last;
        };
      }
      last; 
    } else {
      print("This is a FLATFIELD image\n") if $options{d};
      $HeadingLabel="<h3>CALIBRATION exposures</h3>";
      $SectionLabel="<h4>FlatFields</h4>";
      $options{t}="FLAT";
      SWITCH: for ( $options{a} ) {

        /mklink/    && do {
          $fileinfo="<tt><a href=\"${DSoN}/${arcfilebis}/index.html\">$arcfile</a> == $origname == ${fn}</tt><br>";
          last; 
        };

        /add/    && do {
          $fileinfo="<tt>$arcfile == $origname == ${fn}</tt><br>";
          last;
        };
      }
      last;
    }
  };


  /WAVE|CALIBRATION/    && do {
    print("This is a CALIBRATION image\n") if $options{d};
    $HeadingLabel="<h3>CALIBRATION exposures</h3>";
    $SectionLabel="<h4>Wavelength Calibrations</h4>";
    $options{t}="WAVE";
      SWITCH: for ( $options{a} ) {

        /mklink/    && do {
          $fileinfo="<tt><a href=\"${DSoN}/${arcfilebis}/index.html\">$arcfile</a> == $origname == ${fn}</tt><br>";
          last; 
        };

        /add/    && do {
          $fileinfo="<tt>$arcfile == $origname == ${fn}</tt><br>";
          last;
        };
      }
   last;
   };


   /DRSINIT/    && do {
     print("This is a DRS Initialisation...\n") if $options{d};
     $HeadingLabel="<h3>DRS Initialisation</h3>";
     $SectionLabel="<h4></h4>";

     SWITCH: for ( $options{a} ) {

        /mklink/    && do {
          $fileinfo="<tt><a href=\"${DSoN}/${arcfilebis}/index.html\">${arcfilebis}</a> :: FF = $FFim :: WLC = $WLCim</tt><br>";
          $logInfo="First FF <tt>$FFfimArcName&nbsp; == $FFfimOrigname == $FFfim</tt><br>First WLC <tt>$WLCfimArcName&nbsp; == $WLCfimOrigname == $WLCfim</tt><br>Guess session = $options{G}";
          last;
        };

        /add/    && do {
          $fileinfo="<tt>${arcfilebis} :: FF = $FFim :: WLC = $WLCim</tt><br>";
          $logInfo="First FF <tt>$FFfimArcName&nbsp; == $FFfimOrigname == $FFfim</tt><br>First WLC <tt>$WLCfimArcName&nbsp; == $WLCfimOrigname == $WLCfim</tt><br>Guess session = $options{G}";
          last;
        };
     }
   last;
   };


   /OBJ,SKY|OBJECT,SKY|SKY,OBJECT/ && do {
     print("This is a OBJECT-SKY image\n") if $options{d}; 
     $HeadingLabel="<h3>SCIENCE exposures</h3>";
     $SectionLabel="<h4>OBJECT-SKY</h4>";
     $options{t}="OBJ,SKY";

      SWITCH: for ( $options{a} ) {

        /mklink/    && do {
          $fileinfo="<tt><a href=\"${DSoN}/${arcfilebis}/index.html\">$arcfile</a> == $origname == ${fn}</tt><br>";
          last; 
        };

        /add/    && do {
          $fileinfo="<tt>$arcfile == $origname == ${fn}</tt><br>";
          last;
        };
      }
   last; 
  };

   /OBJ,CAL|OBJECT,WAVE|OBJECT,FLAT|WAVE,OBJECT|FLAT,OBJECT/ && do {
     print("This is a OBJECT-CALIBRATION image\n") if $options{d}; 
     $HeadingLabel="<h3>SCIENCE exposures</h3>";
     $SectionLabel="<h4>OBJECT-CALIBRATION</h4>";
     $options{t}="OBJ,CAL";

       SWITCH: for ( $options{a} ) {

        /mklink/    && do {
          $fileinfo="<tt><a href=\"${DSoN}/${arcfilebis}/index.html\">$arcfile</a> == $origname == ${fn}</tt><br>";   
          last; 
        };

        /add/    && do {
          $fileinfo="<tt>$arcfile == $origname == ${fn}</tt><br>";
          last; 
        };
       }
   last; 
  };
   /STD,SKY|SKY,STD|FLUX,SKY|SKY,FLUX|VELOC,SKY|SKY,VELOC|TELLURIC,SKY|SKY,TELLURIC|SKY,SKY/ && do {
     print("This is a STD-SKY image\n") if $options{d}; 
     $HeadingLabel="<h3>CALIB exposures</h3>";
     $SectionLabel="<h4>STD-SKY</h4>";
     $options{t}="STD,SKY";

      SWITCH: for ( $options{a} ) {

        /mklink/    && do {
          $fileinfo="<tt><a href=\"${DSoN}/${arcfilebis}/index.html\">$arcfile</a> == $origname == ${fn}</tt><br>";
          last; 
        };

        /add/    && do {
          $fileinfo="<tt>$arcfile == $origname == ${fn}</tt><br>";
          last;
        };
      }
   last; 
  };

   /STD,WAVE|STD,FLAT|WAVE,STD|FLAT,STD|FLUX,WAVE|WAVE,FLUX|VELOC,WAVE|WAVE,VELOC|TELLURIC,WAVE|WAVE,TELLURIC|SKY,WAVE|WAVE,SKY|FLUX,FLAT|FLAT,FLUX|VELOC,FLAT|FLAT,VELOC|TELLURIC,FLAT|FLAT,TELLURIC|SKY,FLAT|FLAT,SKY/ && do {
     print("This is a STD-CALIBRATION image\n") if $options{d}; 
     $HeadingLabel="<h3>CALIB exposures</h3>";
     $SectionLabel="<h4>STD-CALIBRATION</h4>";
     $options{t}="STD,CAL";

       SWITCH: for ( $options{a} ) {

        /mklink/    && do {
          $fileinfo="<tt><a href=\"${DSoN}/${arcfilebis}/index.html\">$arcfile</a> == $origname == ${fn}</tt><br>";   
          last;
        };

        /add/    && do {
          $fileinfo="<tt>$arcfile == $origname == ${fn}</tt><br>";
          last; 
        };
       }
   last; 
  };
  die "Unknown value for EXPTYPE: $options{t}";
}


print("options{t} : $options{t}\n") if $options{d};
print("options{a} : $options{a}\n") if $options{d};
print("\n") if $options{d};

SWITCH: for ( $options{a} ) {
  
  /add/ && do {
   addORmklink();
   last; 
  };


  /display/ && do {
    system("ferosShowWebPage -P FEROS-QC http://www.ls.eso.org/lasilla/sciops/2p2/E2p2M/FEROS/QC/ReferenceDRSInits/".${gain}."-".${WLCroot}."/index.html") if $options{r};
    $ofile="$ENV{FEROS_DRS}/html/${DSoN}/${arcfilebis}/index.html";
    sleep(5);
    system("ferosShowWebPage -P FEROS-QC file://".${ofile});

   last;
  };

  /mklink/ && do {
   addORmklink();

#### Create the reduced image webpage (index)
   $ofile="$ENV{FEROS_DRS}/html/${DSoN}/${arcfilebis}/index.html";

## Check if the page already exists If it does, it's going to be deleted
    `rm $ofile` if ( -e $ofile );
    if ( $options{t} =~ /DRSINIT/ ) {
      copy ("$ENV{FEROS_DRS}/html/FEROS-Template-DRSINIT.html",$ofile) or die "Can't create webpage $ofile";
    } else {
      copy ("$ENV{FEROS_DRS}/html/FEROS-Template-DRS-RedImg.html",$ofile) or die "Can't create webpage $ofile";
    }

    open( FILE , "$ofile"  );
    @ifile = <FILE>;
    close(FILE);

    if ( $options{t} =~ /DRSINIT/ ) {
      open( FILE , "> $ofile" );
      my $savefn=$fn;
## Add list of FFs
      foreach $j ( @FFfiles ) {
        $file=substr($j,0,11);
        $fn=basename($file);
        chomp($options{t}=`gethead $file 'HIERARCH ESO DPR TYPE'`);
        $fileinfo=mkFileInfo("add");
        $logInfo=mkLogInfo();
        addORmklink();
      }
## Add list of WLCs
      foreach $j ( @WLCfiles ) {
        $file=substr($j,0,11);
        $fn=basename($file);
        chomp($options{t}=`gethead $file 'HIERARCH ESO DPR TYPE'`);
        $fileinfo=mkFileInfo("add");
        $logInfo=mkLogInfo();
        addORmklink();
      }
      $fn=$savefn;
      foreach (@ifile) {
        s,&lt;Subject&gt;,DRS $arcfile,g;
        s,&lt;arcname&gt;,$arcfile,g; 
        s,&lt;Info&gt;,FF catalogue = <tt>$options{F}</tt> :: WLC catalogue <tt>$options{W}</tt> :: Guess session = <tt>$options{G}</tt><br>\nFF Image = <tt>$FFim</tt> :: WLC Image = <tt>$WLCim</tt><br>\nFirst FF <tt>$FFfimArcName&nbsp; == $FFfimOrigname == $FFfim</tt><br>First WLC <tt>$WLCfimArcName&nbsp; == $WLCfimOrigname == $WLCfim</tt><br>,g;
        print FILE;
      }
      close(FILE);


################################################################################
## Master FLAT Image view...
## Don't use ${FFfimArcName} as this gets the image of the first frame, not the Master FLAT image...
      @plotlis=(glob("${dir}/$FFim.ps*")); # , glob("${dir}/".basename(${FFfimArcName},".fits").".ps*"));
      if ( @plotlis != 0 ) {
        foreach $i ( @plotlis ) {
          $i = basename($i,".ps");
          $ibis = $i;
          $ibis =~ s/:/_/;
          print("Converting $i... ") if $options{d};
          system("convert ${dir}/${i}.ps $ENV{FEROS_DRS}/html/${DSoN}/${arcfilebis}/${ibis}.png $bgamp") ;
          print("Master FLAT image PS-->PNG conversion complete: ${ibis}.png\n") if $options{d} ;
          open( FILE , "> $ofile" );
          foreach (@ifile) {
            s|<!-- Add MasterFLAT Im PNG here -->|<a href="${ibis}.png"><img src="${ibis}.png" width="183" height="330" alt="${ibis}.png" align="right"></a>|g;
            print FILE;
          }
          close(FILE);
        }
      } else {
        print("Master FLAT image PS file not found.\n") if $options{d};
      }
################################################################################
## Master WAVE Image view...
## Don't use ${WLCfimArcName} as this gets the image of the first frame, not the Master WAVE image...
      @plotlis=(glob("${dir}/$WLCim.ps*")); # , glob("${dir}/".basename(${WLCfimArcName},".fits").".ps*"));
      if ( @plotlis != 0 ) {
        foreach $i ( @plotlis ) {
          $i = basename($i,".ps");
          $ibis = $i;
          $ibis =~ s/:/_/;
          print("Converting $i... ") if $options{d};
          system("convert ${dir}/${i}.ps $ENV{FEROS_DRS}/html/${DSoN}/${arcfilebis}/${ibis}.png $bgamp") ;
          print("Master WAVE image PS-->PNG conversion complete: ${ibis}.png\n") if $options{d} ;
          open( FILE , "> $ofile" );
          foreach (@ifile) {
            s|<!-- Add MasterWAVE Im PNG here -->|<a href="${ibis}.png"><img src="${ibis}.png" width="183" height="330" alt="${ibis}.png" align="right"></a>|g;
            print FILE;
          }
          close(FILE);
        }
      } else {
        print("Master WAVE image PS file not found.\n") if $options{d};
      }
################################################################################
## Wavelength Solution RMS plots...
      my @plotlis=( glob("${dir}/$WLCim-$options{G}-rms-*.ps"), glob("${dir}/".basename(${WLCfimArcName},".fits")."-$options{G}-rms-*.ps") );
      foreach $i ( @plotlis ) {
        $i = basename($i,".ps");
        $ibis = $i;
        $ibis =~ s/:/_/;
        copy ("${dir}/${i}.ps", "$ENV{FEROS_DRS}/html/${DSoN}/${arcfilebis}/${i}.ps");
        system("convert -rotate 90 $ENV{FEROS_DRS}/html/${DSoN}/${arcfilebis}/${i}.ps $ENV{FEROS_DRS}/html/${DSoN}/${arcfilebis}/${ibis}.png $bgamp") ;
#        ${sysexec}("convert -verbose -rotate 90 $ENV{FEROS_DRS}/html/${DSoN}/${arcfilebis}/${i}.ps $ENV{FEROS_DRS}/html/${DSoN}/${arcfilebis}/${ibis}.png ; \rm -f $ENV{FEROS_DRS}/html/${DSoN}/${arcfilebis}/${i}.ps}") ;
        print("Wavelength Solution RMS plot conversion complete: ${ibis}.png\n") if $options{d} ;
        open( FILE , "> $ofile" );
        foreach (@ifile) {
          s|<!-- Add next WaveSolnRMS plot here -->|<img src="${ibis}.png" width="782" height="542" alt="${ibis}.png" >\n<!-- Add next WaveSolnRMS plot here -->|g;
          print FILE;
        }
        close(FILE);
      }

## OrderTrace plot...
      my @plotlis=( glob("${dir}/$WLCim-$options{G}-OrderTrace*.ps"), glob("${dir}/".basename(${WLCfimArcName},".fits")."-$options{G}-OrderTrace*.ps") );
      if ( @plotlis != 0 ) {
        foreach $i ( @plotlis ) {
          $i = basename($i,".ps");
          $ibis = $i;
          $ibis =~ s/:/_/;
          system("convert ${dir}/${i}.ps $ENV{FEROS_DRS}/html/${DSoN}/${arcfilebis}/${ibis}.png $bgamp") ;
          print("Order Trace plot conversion complete: ${ibis}.png\n") if $options{d} ;
          open( FILE , "> $ofile" );
          foreach (@ifile) {
            s|<!-- <li><a href="#Tag:OrderTrace">Order Trace</a></li> -->|<li><a href="#Tag:OrderTrace">Order Trace</a></li>|g;
            s|<!-- <h4>Order Trace</h4> -->|<h4>Order Trace</h4>|g;
            s|<!-- Add next OrderTrace plot here -->|<img src="${ibis}.png" width="542" height="782" alt="${ibis}.png" >\n<!-- Add next OrderTrace plot here -->|g;
            print FILE;
          }
          close(FILE);
        }
      }

## Profile template plot...
      @plotlis=(glob("${dir}/*${FFim}_templatet.ps"), glob("${dir}/*".basename("${FFfimArcName}",".fits")."_tpl.ps") );
      if ( @plotlis != 0 ) {
        foreach $i ( @plotlis ) {
          $i = basename($i,".ps");
          $ibis = $i;
          $ibis =~ s/:/_/;
          system("convert ${dir}/${i}.ps $ENV{FEROS_DRS}/html/${DSoN}/${arcfilebis}/${ibis}.png $bgamp") ;
          print("Profile template plot conversion complete: ${ibis}.png\n") if $options{d} ;
          open( FILE , "> $ofile" );
          foreach (@ifile) {
            s|<!-- <li><a href="#Tag:TemplateT">Profile template</a></li> -->|<li><a href="#Tag:TemplateT">Profile template</a></li>|g;
            s|<!-- <h4>Profile template</h4> -->|<h4>Profile template</h4>|g;
            s|<!-- Add next templatet plot here -->|<img src="${ibis}.png" width="542" height="782" alt="${ibis}.png" >\n<!-- Add next templatet plot here -->|g;
            print FILE;
          }
          close(FILE);
        }
      }

## FF S/N plots...
      foreach $j ( @FFfiles ) {
        my $fnum=substr($j,4,4);
        my $tfile=substr($j,0,11);
        chomp(my $tarcfile=`gethead ${tfile} ARCFILE`);
        print("Archive file name : $tarcfile  \n") if $options{d};
        my $tarcfilebis=$tarcfile;
        $tarcfilebis =~ s|.fits||g;
        @plotlis=(glob("${dir}/*f${fnum}-snr.ps"), glob("${dir}/*r.${tarcfilebis}_snr.ps") );
        if ( @plotlis != 0 ) {
          foreach $i ( @plotlis ) {
            $i = basename($i,".ps");
            $ibis = $i;
            $ibis =~ s/:/_/;
            system("convert -rotate 90 ${dir}/${i}.ps $ENV{FEROS_DRS}/html/${DSoN}/${arcfilebis}/${ibis}.png $bgamp") ;
            print("FF S/N plot conversion complete: ${ibis}.png\n") if $options{d} ;
            open( FILE , "> $ofile" );
            foreach (@ifile) {
	      s|<!-- <li><a href="#Tag:FFSNRplots">Flatfield Signal to Noise plots</a></li> -->|<li><a href="#Tag:FFSNRplots">Flatfield Signal to Noise plots</a></li>|g;
              s|<!-- <h4>Signal to Noise in FFs</h4> -->|<h4>Signal to Noise in FFs</h4>|g;
              s|<!-- Add next FFSNR plot here -->|<img src="${ibis}.png" width="782" height="542" alt="${ibis}.png" >\n<!-- Add next FFSNR plot here -->|g;
              print FILE;
            }
            close(FILE);
          }
        }
      }

## FF ratio plots...
      foreach $j ( @FFfiles ) {
        my $fnum=substr($j,4,4);
        my $tfile=substr($j,0,11);
        chomp(my $tarcfile=`gethead ${tfile} ARCFILE`);
        print("Archive file name : $tarcfile  \n") if $options{d};
        my $tarcfilebis=$tarcfile;
        $tarcfilebis =~ s|.fits||g;
        @plotlis=(glob("${dir}/*f${fnum}o*ext*.ps"), glob("${dir}/*r.${tarcfilebis}o*ext*.ps") );
        if ( @plotlis != 0 ) {
          foreach $i ( @plotlis ) {
            $i = basename($i,".ps");
            $ibis = $i;
            $ibis =~ s/:/_/;
            copy ("${dir}/${i}.ps", "$ENV{FEROS_DRS}/html/${DSoN}/${arcfilebis}/${i}.ps");
            system("convert $ENV{FEROS_DRS}/html/${DSoN}/${arcfilebis}/${i}.ps $ENV{FEROS_DRS}/html/${DSoN}/${arcfilebis}/${ibis}.png $bgamp") ;
            print("FF ratio plot conversion complete: ${ibis}.png\n") if $options{d} ;
            open( FILE , "> $ofile" );
            foreach (@ifile) {
	      s|<!-- <li><a href="#Tag:FFRatioplots">Flatfield ratio plots</a></li> -->|<li><a href="#Tag:FFRatioplots">Flatfield ratio plots</a></li>|g;
              s|<!-- <h4>FF ratio plots</h4> -->|<h4>FF ratio plots</h4>|g;
              s|<!-- Add next FFRatio plot here -->|<img src="${ibis}.png" width="542" height="782" alt="${ibis}.png" >\n<!-- Add next FFRatio plot here -->|g;
              print FILE;
            }
            close(FILE);
          }
        }
      }

## WLC focus plots...
      foreach $j ( @WLCfiles ) {
        my $fnum=substr($j,4,4);
        my $tfile=substr($j,0,11);
        chomp(my $tarcfile=`gethead ${tfile} ARCFILE`);
        print("Archive file name : $tarcfile  \n") if $options{d};
        my $tarcfilebis=$tarcfile;
        $tarcfilebis =~ s|.fits||g;
        @plotlis=(glob("${dir}/*f${fnum}-focus.ps"), glob("${dir}/*r.${tarcfilebis}-focus.ps") );
        if ( @plotlis != 0 ) {
          foreach $i ( @plotlis ) {
            $i = basename($i,".ps");
            $ibis = $i;
            $ibis =~ s/:/_/;
            system("convert -rotate 90 ${dir}/${i}.ps $ENV{FEROS_DRS}/html/${DSoN}/${arcfilebis}/${ibis}.png $bgamp") ;
            print("WLC focus plot conversion complete: ${ibis}.png\n") if $options{d} ;
            open( FILE , "> $ofile" );
            foreach (@ifile) {
	      s|<!-- <li><a href="#Tag:WLCfocusplots">Wavelength Calibration focus plots</a></li> -->|<li><a href="#Tag:WLCfocusplots">Wavelength Calibration focus plots</a></li>|g;
              s|<!-- <h4>WLC focus plots</h4> -->|<h4>WLC focus plots</h4>|g;
              s|<!-- Add next WLCfocus plot here -->|<img src="${ibis}.png" width="782" height="542" alt="${ibis}.png" >\n<!-- Add next WLCfocus plot here -->|g;
              print FILE;
            }
            close(FILE);
          }
        }
      }


    } else {
## FIBRE 1 :
## Look if postscript plots of spectra exist for this science image. If they do, it converts them in .png and into the file diretory ##
    
################################################################################
## Raw Image view...
    @plotlis=(glob("${dir}/$fps.ps*"), glob("${dir}/${arcfilebis}.ps*"));
    if ( @plotlis != 0 ) {
      foreach $i ( @plotlis ) {
        $i = basename($i,".ps");
        $ibis = $i;
        $ibis =~ s/:/_/;
        print("Converting ${i}.ps... ") if $options{d};
        system("convert ${dir}/${i}.ps $ENV{FEROS_DRS}/html/${DSoN}/${arcfilebis}/${ibis}.png $bgamp") ;
        print("Raw image PS-->PNG conversion complete: ${ibis}.png\n") if $options{d} ;
        open( FILE , "> $ofile" );
        foreach (@ifile) {
          s|<!-- Add RAW Im PNG here -->|<a href="${ibis}.png"><img src="${ibis}.png" width="183" height="330" alt="${ibis}.png" align="right"></a>|g;
          print FILE;
        }
        close(FILE);
      }
    } else {
      print("Raw image PS file not found.\n") if $options{d};
    }
    foreach $fibre ( "1", "2" ) {
      my $fp=$fn;
      $fp =~ s|fero|f|g;
      $fp =~ s|.mt|${fibre}|g;
      my $fps=$fn;
      $fps =~ s|.mt||g;
################################################################################
## Combined spectra...
      @plotlis=(glob("${dir}/*$fp.ps"), glob("${dir}/*r.${arcfilebis}.${fibre}.ps"), glob("${dir}/*$fp-*.ps"), glob("${dir}/r.${arcfilebis}.${fibre}-*.ps"));
      if ( @plotlis != 0 ) {
        foreach $i (@plotlis) {
          print("Converting $i... ") if $options{d};
          chomp($i);
          my $ibis=$i;
          $ibis =~ s/:/_/g;;
          $ibis =~ s|${dir}/||g;
          $ibis =~ s|.ps||g;
          system("convert -rotate 90 ${i}  $ENV{FEROS_DRS}/html/${DSoN}/${arcfilebis}/$ibis.png $bgamp") ;
          print("Fibre ${fibre} : Spectrum conversion complete: $ibis.png\n") if $options{d} ; 
        }

##Play with strings to print out the .png plots on the index webpage
        open( FILE , "> $ofile" );
        @plotlis2=@plotlis;
        foreach (@plotlis2) {
          s|${dir}/|${DSoN}/${arcfilebis}/|g;
          s|\.ps|\.png|g;
        }
        foreach (@ifile) {
          s|<li>Fibre ${fibre} : No Plots for this spectrum</li>|<li><a href="#Tag:Fib${fibre}">Fibre ${fibre}</a>|g;
          s|<!--${fibre} <h3>Fibre-${fibre} \[Object Fibre\]</h3> ${fibre}-->|<h3> Fibre-${fibre} \[Object Fibre\]</h3>|g;
          s|<!--${fibre} <h3>Fibre-${fibre} \[Sky/Calibration Fibre\]</h3> ${fibre}-->|<h3> Fibre-${fibre} \[Sky/Calibration Fibre\]</h3>|g;
          s|<!--${fibre} <h4>Plots of reduced spectra</h4> ${fibre}-->|<h4>Plots of reduced spectra</h4>|g;
          $lplotlis2=length(@plotlis2);
          foreach $m (@plotlis2) {
            chomp($m);
            my $mbis=$m;
            $mbis =~ s|${DSoN}/${arcfilebis}/||g;
            $mbis =~ s/:/_/g;;
            chomp($mbis);
            s|<!-- Add next RedSpecFib${fibre} plot here -->|<a href="${mbis}"><img src="${mbis}" width="782" height="542" alt="${mbis}" ></a>\n<!-- Add next RedSpecFib${fibre} plot here -->|g;
          }
        }
        close(FILE);
      } else {
        print("Fibre ${fibre} : No files ${dir}/*$fp.ps ${dir}/*$fp-*.ps found.\n") if $options{d};
      }
################################################################################
## OrderByOrder Plots spectra...
      @plotlis=(glob("${dir}/*$fp????.ps"), glob("${dir}/*r.${arcfilebis}.*.${fibre}????.ps"));
      if ( @plotlis != 0 ) {
        foreach $i (@plotlis) {
          print("Converting $i... ") if $options{d};
          chomp($i);
          my $ibis=$i;
          $ibis =~ s/:/_/g;;
          $ibis =~ s|${dir}/||g;
          $ibis =~ s|.ps||g;
          system("convert -rotate 90 ${i}  $ENV{FEROS_DRS}/html/${DSoN}/${arcfilebis}/$ibis.png $bgamp") ;
          print("Fibre ${fibre} : Spectrum conversion complete: $ibis.png\n") if $options{d} ; 
        }

##Play with strings to print out the .png plots on the index webpage
        open( FILE , "> $ofile" );
        @plotlis2=@plotlis;
        foreach (@plotlis2) {
          s|${dir}/|${DSoN}/${arcfilebis}/|g;
          s|\.ps|\.png|g;
        }
        foreach (@ifile) {
          s|<li>Fibre ${fibre} : No Plots for this spectrum</li>|<li><a href="#Tag:Fib${fibre}">Fibre ${fibre}</a>|g;
          s|<!--${fibre} <h3>Fibre-${fibre} \[Object Fibre\]</h3> ${fibre}-->|<h3> Fibre-${fibre} \[Object Fibre\]</h3>|g;
          s|<!--${fibre} <h3>Fibre-${fibre} \[Sky/Calibration Fibre\]</h3> ${fibre}-->|<h3> Fibre-${fibre} \[Sky/Calibration Fibre\]</h3>|g;
          s|<!--${fibre} <h4>Plots of reduced spectra, order by order</h4> ${fibre}-->|<h4>Plots of reduced spectra, order by order</h4>|g;
          $lplotlis2=length(@plotlis2);
          foreach $m (@plotlis2) { 
            chomp($m);
            my $mbis=$m;
#	    printf("MBIS = $mbis\n");
            $mbis =~ s|${DSoN}/${arcfilebis}/||g;
#	    printf("MBIS = $mbis\n");
            $mbis =~ s/:/_/g;
#	    printf("MBIS = $mbis\n");
            chomp($mbis);
#	    printf("MBIS = $mbis\n");
            s|<!-- Add next RedSpecObyOFib${fibre} plot here -->|<a href="${mbis}"><img src="${mbis}" width="391" height="271" alt="${mbis}" ></a>\n<!-- Add next RedSpecObyOFib${fibre} plot here -->|g;
#	    printf("<a href=\"${mbis}\"><img src=\"${mbis}\" width=\"391\" height=\"271\" alt=\"${mbis}\" >");
#	    printf("MBIS = $mbis\n");
          }
        }
        close(FILE);
      } else {
        print("Fibre ${fibre} : No Order By Order plots found.\n") if $options{d};
      }
################################################################################
## Efficiency plots
      @plotlis=(glob("${dir}/*efficiency".substr($file,4,4)."${fibre}.ps"), glob("${dir}/*r.${arcfilebis}.${fibre}_eff.ps") );
      if ( @plotlis != 0 ) {
        foreach $i ( @plotlis ) {
          $i = basename($i,".ps");
          $ibis = $i;
          $ibis =~ s/:/_/;
          system("convert -rotate 90 ${dir}/${i}.ps $ENV{FEROS_DRS}/html/${DSoN}/${arcfilebis}/${ibis}.png $bgamp") ;
          print("Efficiencey plot conversion complete: ${ibis}.png\n") if $options{d} ;
          open( FILE , "> $ofile" );
          foreach (@ifile) {
            s|<!-- <li><a href="#Tag:Efficiency">Efficiency plots</a></li> -->|<li><a href="#Tag:Efficiency">Efficiency plots</a></li>|g;
            s|<!-- <h4>Efficiency plots</h4> -->|<h4>Efficiency plots</h4>|g;
            s|<!-- Add next Efficiency plot here -->|<a href="${ibis}.png"><img src="${ibis}.png" width="782" height="542" alt="${ibis}.png" ></a>\n<!-- Add next Efficiency plot here -->|g;
            print FILE;
          }
          close(FILE);
        }
      }
################################################################################
## Response plots
      @plotlis=(glob("${dir}/*response".substr($file,4,4)."${fibre}.ps"), glob("${dir}/*r.${arcfilebis}.${fibre}_res.ps") );
      if ( @plotlis != 0 ) {
        foreach $i ( @plotlis ) {
          $i = basename($i,".ps");
          $ibis = $i;
          $ibis =~ s/:/_/;
          system("convert -rotate 90 ${dir}/${i}.ps $ENV{FEROS_DRS}/html/${DSoN}/${arcfilebis}/${ibis}.png $bgamp") ;
          print("Response Function plot conversion complete: ${ibis}.png\n") if $options{d} ;
          open( FILE , "> $ofile" );
          foreach (@ifile) {
            s|<!-- <li><a href="#Tag:Response">Response Function plots</a></li> -->|<li><a href="#Tag:Response">Response Function plots</a></li>|g;
            s|<!-- <h4>Response Function plots</h4> -->|<h4>Response Function plots</h4>|g;
            s|<!-- Add next Response plot here -->|<a href="${ibis}.png"><img src="${ibis}.png" width="782" height="542" alt="${ibis}.png" ></a>\n<!-- Add next Response plot here -->|g;
            print FILE;
          }
          close(FILE);
        }
      }
## End of Fibres 1 & 2 loop
    }
## Fibreratio plots
    @plotlis=(glob("${dir}/*fiberratio".substr($file,4,4).".ps"), glob("${dir}/*r.${arcfilebis}_rfr.ps") );
    if ( @plotlis != 0 ) {
      foreach $i ( @plotlis ) {
        $i = basename($i,".ps");
        $ibis = $i;
        $ibis =~ s/:/_/;
        system("convert -rotate 90 ${dir}/${i}.ps $ENV{FEROS_DRS}/html/${DSoN}/${arcfilebis}/${ibis}.png $bgamp") ;
        print("Efficiencey plot conversion complete: ${ibis}.png\n") if $options{d} ;
        open( FILE , "> $ofile" );
        foreach (@ifile) {
          s|<!-- <li><a href="#Tag:FibreRatio">Relative Fibre Ratio plots</a></li> -->|<li><a href="#Tag:FibreRatio">Relative Fibre Ratio plots</a></li>|g;
          s|<!-- <h4>Relative Fibre Ratio</h4> -->|<h4>Relative Fibre Ratio</h4>|g;
          s|<!-- Add next FibreRatio plot here -->|<a href="${ibis}.png"><img src="${ibis}.png" width="782" height="542" alt="${ibis}.png" ></a>\n<!-- Add next FibreRatio plot here -->|g;
          print FILE;
        }
        close(FILE);
      }
    }


## TCCD Images :
# Use .info file to determine the appropriate FFHV image it includes in the page and then include it after being converted

    my $arcfileter=$arcfile;
    $arcfileter =~ s|.fits|.info|g;
    $arcfileter =~ s|FEROS|FFHV|g;
    print("arcfileter : $arcfileter\n") if $options{d} ;
    if ( ! -e "$ENV(FEROS_FFHV)/${DSoN}/$arcfileter" ) {
      print("TCCD Images : Info file not found\n") if $options{d} ;
    } else {
      $infofile="$arcfileter";
      open( FILE , "$infofile");
      $info = <FILE>;
      close(FILE);
      my $info2 =$info;
      $info =~ s|.fits|.fits.gz|g;
      $info2=~ s|.fits|TCCD.png|g;
      system("convert -rotate 90 $ENV(FEROS_FFHV)/${DSoN}/$info  $ENV{FEROS_DRS}/html/${DSoN}/${arcfilebis}/$info2 $bgamp");
      print("TCCD Image succesfully converted and loaded\n")  if $options{d} ;
      $lengthinfo=length($info);
      print("lengthinfo is $lengthinfo\n") if $options{d} ;
    }

## By subsitutions again, write on the index webpage the general and the TCCD information of the science image     
    open( FILE , "> $ofile" );
    foreach (@ifile) {
      s,&lt;Subject&gt;,DRS $arcfile,g;
      s,&lt;arcname&gt;,$arcfile,g; 
      s,FEROS.yyyy-mm-ddThh:mm:ss.sss.fits,$arcfile,g;
      s,FEROS_ech_cal_bias_0001.fits,$origname,g;
      s,feroNNNN.mt,$fn,g;
      s|TargetName|$obj|g;
      s|ExpTime|$exptime|g;
      s|ROMMode|$gain|g;
      s|OBID|$obsid|g;
      s|RunID|$progid|g;
      if($lengthinfo !=0  ){
        s|<!-- Add TCCD Image here -->|<img src="${info2}" width="782" height="542" alt="TCCD.png" >|g;
        s|<!-- <h3> TCCD image</h3> -->|<h3> TCCD image</h3>|g; 
        s|<li>TCCD : No image for this spectrum|<li><a href="#Tag:TCCD">TCCD</a>|g;
      }
      print FILE ;
    }
    close(FILE);
    }
    last;
  };
}

sub mkFileInfo  {
  chomp($arcfile=`gethead $file ARCFILE`);
  print("Archive file name : $arcfile  \n") if $options{d};
  chomp($origname=`gethead $file ORIGFILE`);
  $arcfilebis=$arcfile;
  $arcfilebis =~ s|.fits||g;
  my $fileinfo="null";
  SWITCH: for ( @_ ) {

    /mklink/    && do {
      $fileinfo="<tt><a href=\"${DSoN}/${arcfilebis}/index.html\">$arcfile</a> == $origname == ${fn}</tt><br>";
      last; 
    };

    /add/    && do {
      $fileinfo="<tt>$arcfile == $origname == ${fn}</tt><br>";
      last;
    };

    /display/    && do {
      $fileinfo="";
      last;
    };
    die "Unknown value for ACTION: $options{a}";
  }
  return($fileinfo);
}

sub mkLogInfo() {
###Fetch relevant informations from the header
  chomp(my $xbin=`gethead $file 'HIERARCH ESO DET WIN1 BINX'`);
  chomp(my $ybin=`gethead $file 'HIERARCH ESO DET WIN1 BINY'`);
  chomp($gain=`gethead $file 'HIERARCH ESO DET READ CLOCK'`);
  if ($gain !~ /[\w\d]/ ){$gain = "none";}
### As of VLTSW upgrade 2005-12, complete ReadOut Mode is given...
  $gain =~ s|R 60Kps High Gain|60kHz,1,high,${xbin}x${ybin}|g;
  $gain =~ s|R 225Kps Low Gain|225kHz,1,low,${xbin}x${ybin}|g;
  $gain =~ s|R 625Kps Low Gain|625kHz,1,med,${xbin}x${ybin}|g;
  $gain =~ s|R 625Kps Med Gain|625kHz,1,med,${xbin}x${ybin}|g;
  $gain =~ s|R 60Kps High Gai|60kHz,1,high,${xbin}x${ybin}|g;
  $gain =~ s|R 225Kps Low Gai|225kHz,1,low,${xbin}x${ybin}|g;
  $gain =~ s|R 625Kps Low Gai|625kHz,1,med,${xbin}x${ybin}|g;
  $gain =~ s|R 625Kps Med Gai|625kHz,1,med,${xbin}x${ybin}|g;
  chomp($obsid=`gethead $file 'HIERARCH ESO OBS ID'`);
  chomp($progid=`gethead $file 'HIERARCH ESO OBS PROG ID'`);
  chomp($obj=`gethead $file 'HIERARCH ESO OBS TARG NAME'`);
  chomp($obj=`gethead $file 'OBJECT'`) if ( $obj !~ /[\w\d]/ );
  chomp($exptime=`gethead $file 'EXPTIME'`);

###Store these particurlarly relevant informations into the logInfo string
  my $logInfo = sprintf("%-11d %-14s %-30s %8.2f %-30s ",$obsid,$progid,$obj,$exptime,$gain);
  $logInfo =~ s, ,&nbsp;,g;
  return($logInfo);

}

sub addORmklink {

  my $gr = grep( /$fileinfo/ , @ifile );
  if ( $gr != 0 ) { 
    print("Warning : This file has already been processed...\n") if $options{d};
  } else {
    $gr = grep( /${arcfilebis}/ , @ifile );
    if ( $gr == 0 ) {
### Write the information with substitutions onto the Master Webpage
      print("Adding entry to $ofile...\n") if $options{d};
      print("$fileinfo\n") if $options{d};
      print("HeadingLabel = $HeadingLabel\n") if $options{d};
      print("SectionLabel = $SectionLabel\n") if $options{d};
      print("Add next $options{t} image here\n") if $options{d};
      open( FILE , "> $ofile" );
      foreach (@ifile) {
        s,&lt;Subject&gt;,DRS $DSoN,g;
        s,yyyy-mm-dd,$DSoN,g;
        s,<!-- ${HeadingLabel} -->,${HeadingLabel},g;
        s,<!-- ${SectionLabel} -->,${SectionLabel},g;
        s,<!-- Add next $options{t} image here -->,${fileinfo}\n<tt><small>${logInfo}</small></tt><br>\n<!-- Add next $options{t} image here -->,g ;
        print FILE ;
      }
      close(FILE);
    } else {
      if ( $options{a} =~ /mklink/ ) {
### This part deals with the creation of the link for a file already added
        $gr = grep( /${DSoN}\/${arcfilebis}\/index.html/ , @ifile );
        if ( $gr == 0 ) {
          print("Adding link...\n") if $options{d};
          open( FILE , "> $ofile" );
          foreach (@ifile) {
            s,$arcfile,<a href=\"${DSoN}/${arcfilebis}/index.html\">$arcfile</a>,g ;
            print FILE;
          }
          close(FILE);
        } else {
          print("Warning : This file has already been processed...\n") if $options{d};
        }
      }
    }
  }
}
