! "@(#) $Id: FEROSCTcomptime.prg,v 1.4 2003/10/29 22:59:03 jpritcha Exp $"
!+++++++++++++++++++++++++++++++++++++++++++++++++++++++
!
!.COPYRIGHT   (C) 2002 European Southern Observatory
!.IDENT       FEROSCTcomptime.prg
!.AUTHOR      John Pritchard, la Silla/ESO
!.KEYWORDS    Spectroscopy, Echelle, FEROS
!.PURPOSE     Command @@ FEROSCTcomptime <frame>
!.VERSION     1.0    Creation   2002-08-28
!
!-------------------------------------------------------
!
DEFINE/PARAMETER P1 ?        CHAR "Image : "
DEFINE/PARAMETER P2 854,2125 CHAR "Window Center : "
DEFINE/PARAMETER P3 17,12    CHAR "Window Half Sizes : "
DEFINE/PARAMETER P4 -1.      CHAR "Ref Flux Level : "
!
DEFINE/LOCAL I/I/1/1        0
DEFINE/LOCAL IEXPTYPE/I/1/1 1
!
DEFINE/LOCAL EXPTIME/R/1/1  0.
DEFINE/LOCAL CG1/R/1/14     0. A
DEFINE/LOCAL CG2/R/1/14     0. A
DEFINE/LOCAL CGIGC/R/1/2    12.5,34.5
DEFINE/LOCAL CGWIND/R/1/1   30.0
DEFINE/LOCAL IL/R/1/10      0. A
DEFINE/LOCAL IL1/R/1/10     0. A
DEFINE/LOCAL IL2/R/1/10     0. A
DEFINE/LOCAL RELFFLUX/R/1/1 0.
DEFINE/LOCAL RELRFLUX/R/1/1 0.
DEFINE/LOCAL MPBC/R/1/1     0.
DEFINE/LOCAL REQETIME/R/1/1 0.
DEFINE/LOCAL SFPIX/R/1/2    0. A
!
DEFINE/LOCAL WINCENT/I/1/2  {P2}
DEFINE/LOCAL WINSF/I/1/4    0 A
DEFINE/LOCAL WINHS/I/1/2    {P3}
!
!CGIGC(1) = (WINHS(1)+1.0)/2.0
!CGIGC(2) = WINHS(1)+(WINHS(1)+1.0)/2.0
!
! Reference Flux levels
!    (1) is for CALIBRATION images
!    (2) is for FLATFIELD images
! The following reference fluxes are derived from the study by Pritchard & Faure
! of the data obtained 2001-08-31.
IF {P4} .LT. 0. THEN
  DEFINE/LOCAL REFFLUX/R/1/3 1.19032e+06,9.80483e+06,1.0
ELSE
  DEFINE/LOCAL REFFLUX/R/1/3 {P4} A
ENDIF
!
DEFINE/LOCAL IMG/C/1/80      "{P1}a"
DEFINE/LOCAL EXPTYPE/C/1/30  " "
!
! Get parameters from image...
I = M$EXISTD("{P1}.mt","EXPTYPE")
IF I .EQ. 1 THEN
  WRITE/OUT "{P1}.mt seems to be a BIAS generated image..."
  EXPTYPE = "{{P1}.mt,EXPTYPE}"
  IF "{EXPTYPE}" .EQ. "CALIBRATION" IEXPTYPE = 1
  IF "{EXPTYPE}" .EQ. "FLATFIELD"   IEXPTYPE = 2
  IF "{EXPTYPE}" .EQ. "SCIENCE"     IEXPTYPE = 3
  IF IEXPTYPE .EQ. 0 THEN
    WRITE/OUT "ERROR!!! Exposure type {EXPTYPE} NOT supported"
    GOTO AORR
  ENDIF
ELSE
  WRITE/OUT "{P1}.mt seems to be a FIERA generated image..."
  if "{{P1}.mt,ESO.DPR.CATG}" .eq. "SCIENCE"  then
    EXPTYPE = "SCIENCE"
    IEXPTYPE = 3
  elseif "{{P1}.mt,ESO.DPR.TYPE}" .eq. "DARK"  then
    EXPTYPE = "DARK"
  elseif "{{P1}.mt,ESO.DPR.TYPE}" .eq. "BIAS"  then
    EXPTYPE = "DARK"
  elseif "{{P1}.mt,ESO.DPR.TYPE}" .eq. "FLAT" then
    EXPTYPE = "FLATFIELD"
    IEXPTYPE = 2
  elseif "{{P1}.mt,ESO.DPR.TYPE}" .eq. "WAVE" then
    EXPTYPE = "CALIBRATION"
    IEXPTYPE = 1
  else
    WRITE/OUT "Warning!!! Exposure type NOT recogonised."
    goto exit
  endif
! In FIERA images world coords differ from frame by 50 pixels
!CRVAL1  =                  1.0          / value of ref pixel                    
!CRVAL2  =                  1.0          / value of ref pixel                    
!CRPIX1  =                 51.0          / Ref. pixel of center of rotation      
!CRPIX2  =                  1.0          / Ref. pixel of center of rotation      
!CDELT1  =                  1.0          / Binning factor                        
!CDELT2  =                  1.0          / Binning factor                        
  WINCENT(1) = WINCENT(1) - 50
ENDIF
EXPTIME = M$VALUE({P1}.mt,O_TIME(7))
!
! Compress 2-d image into 1-d...
WINSF(1) = WINCENT(1) - WINHS(1)
WINSF(2) = WINCENT(2) - WINHS(2)
WINSF(3) = WINCENT(1) + WINHS(1)
WINSF(4) = WINCENT(2) + WINHS(2)
AVERAGE/ROW {IMG} = {P1}.mt[{WINSF(1)},{WINSF(2)}:{WINSF(3)},{WINSF(4)}] ? SUM
!
! Get more parameters from image...
SFPIX(1) = M$VALUE({IMG},START(1))
SFPIX(2) = SFPIX(1) + M$VALUE({IMG},NPIX(1)) - 1.
!
! Find centers of the two fibres...
CENTER/GAUSS {IMG},{CGIGC(1)},0 ? ? ? {CGWIND},0
COPY/KEYWORD OUTPUTR/R/1/14 CG1/R/1/14
CENTER/GAUSS {IMG},{CGIGC(2)},0 ? ? ? {CGWIND},0
COPY/KEYWORD OUTPUTR/R/1/14 CG2/R/1/14
!
! Calculate mid-point between the two centres...
MPBC = ( CG2(5) + CG1(5) ) / 2.
!
! Integrate the flux...
! ...individual fibres...
INTEGRATE/LINE {IMG} ? ? ? B {SFPIX(1)},{MPBC},0.05
COPY/KEYWORD OUTPUTR/R/1/10 IL1/R/1/10
INTEGRATE/LINE {IMG} ? ? ? B {MPBC},{SFPIX(2)},0.05
COPY/KEYWORD OUTPUTR/R/1/10 IL2/R/1/10
! Relative Fibre Flux
COMP/KEYWORD RELFFLUX = {IL1(8)}/{IL2(8)}
! ...both fibres...
INTEGRATE/LINE {IMG} ? ? ? B {SFPIX(1)},{SFPIX(2)},0.05
COPY/KEYWORD OUTPUTR/R/1/10 IL/R/1/10
!
!
! Calculate exposure time required to obtain reference flux level
REQETIME = EXPTIME * REFFLUX({IEXPTYPE}) / IL(8)
! Relative Reference Flux...
RELRFLUX = IL(8) / REFFLUX({IEXPTYPE})
IF IEXPTYPE .EQ. 3 THEN
  REQETIME = EXPTIME * REFFLUX({IEXPTYPE}) / IL1(8)
  ! Relative Reference Flux...
  RELRFLUX = IL1(8) / REFFLUX({IEXPTYPE})
ENDIF
WRITE/OUT "================================================================================"
SET/FORMAT I4 F5.1
WRITE/OUT "                  Window Center: ({WINCENT(1)},{WINCENT(2)})"
WRITE/OUT "                         Window: [{WINSF(1)},{WINSF(2)}:{WINSF(3)},{WINSF(4)}]"
WRITE/OUT "           Fibre Center Guesses: ({CGIGC(1)},{CGIGC(2)})"
WRITE/OUT "                  Exposure type: {EXPTYPE}"
SET/FORMAT E12.6
WRITE/OUT "                 REFERENCE Flux: {REFFLUX({IEXPTYPE})}"
WRITE/OUT "   Total Flux in image {P1}: {IL(8)}"
WRITE/OUT " Fibre 1 Flux in image {P1}: {IL1(8)}"
IF IEXPTYPE .NE. 3 WRITE/OUT " Fibre 2 Flux in image {P1}: {IL2(8)}"
SET/FORMAT F6.3
IF IEXPTYPE .NE. 3 WRITE/OUT "Relative FibFlux image {P1}: {RELFFLUX}"
WRITE/OUT "Relative RefFlux image {P1}: {RELRFLUX}"
SET/FORMAT F4.1
WRITE/OUT " Required {EXPTYPE} expoure time: {REQETIME} sec"
WRITE/OUT "{P1} {EXPTYPE}: {REQETIME} sec : {REFFLUX({IEXPTYPE})}: {IL(8)} : {EXPTIME}" >>etimes
SET/FORMAT
WRITE/OUT "================================================================================"

AORR:
