! %W% (ESO-IPG) %G% %U%
! "@(#) $Id: FFsnr.prg,v 1.10 2005/11/21 21:25:26 jpritcha Exp $"
!==========================================================================
!
!    FFsnr.prg
!    J.Pritchard, ESO-LS, 2004-11-16
!
!    Compute and S/N ratios in FFs order by order centered on the blaze
!    wavelength of each order
!
!    Syntax:  @@ FFsnr <fileNumber> [OrderRange (1,39)] [WindowSize (10.)]
!                      [Verbose (no)] [Plots (N)] [PSfiles (N)]
!                      [filePrefix (f)]
!                      [Final/Working Graph Window IDs (5,9)]
!
!###########################################################################

define/para P1 ? I "Filenumber : "
define/para p2 1,39 C "Order range : "
define/para p3 10. N "Window size : "
define/para p4 no C "Display verbose output [no/yes] : "
define/par p5 N     C "Plot : "
define/par p6 N     C "Make PS files : "
define/par p7 f     C "filePrefix (f) : "
define/par p8 5,9   C "Final/Working Graph Window IDs (5,9) : "

define/local snr1/r/1/1 0. All
define/local snr2/r/1/1 0. All
define/local snrr/r/1/1 0.

define/local i/i/1/1 0
define/local j/i/1/1 0
define/local osf/i/1/2  {p2}
define/local iord/i/1/1 0
define/local mord/i/1/1 0
define/local gwids/i/1/2   {p8}

define/local winsize/r/1/1 {p3}
define/local winsf/r/1/2 0.,0.
define/local midwin/r/1/39 0. All
!! For order#1 --> m=63 central wavelength is closer to 3605 than 3600,
!! but the max WL of the useful range for fibre 2 is often less than 3610
!! therefore by using midwin(01)=3600 the range is 3595-3605 which is much
!! more often good and does not result in spuriously low S/R estimations.
midwin(01) = 3600.
midwin(02) = 3670.
midwin(03) = 3735.
midwin(04) = 3795.
midwin(05) = 3855.
midwin(06) = 3920.
midwin(07) = 3995.
midwin(08) = 4055.
midwin(09) = 4125.
midwin(10) = 4205.
midwin(11) = 4285.
midwin(12) = 4365.
midwin(13) = 4455.
midwin(14) = 4540.
midwin(15) = 4630.
midwin(16) = 4725.
midwin(17) = 4825.
midwin(18) = 4930.
midwin(19) = 5040.
midwin(20) = 5155.
midwin(21) = 5275.
midwin(22) = 5405.
midwin(23) = 5530.
midwin(24) = 5665.
midwin(25) = 5815.
midwin(26) = 5970.
midwin(27) = 6130.
midwin(28) = 6295.
midwin(29) = 6475.
midwin(30) = 6670.
midwin(31) = 6865.
midwin(32) = 7080.
midwin(33) = 7325.
midwin(34) = 7545.
midwin(35) = 7795.
midwin(36) = 8075.
midwin(37) = 8365.
midwin(38) = 8665.
midwin(39) = 9025.

define/local OFLABEL/C/1/100 " "

i = m$existk("ARCFILE")
if i .eq. 0 DEFINE/LOCAL ARCFILE/C/1/100 " " ? +lower_levels
ARCFILE = "fero{p1}.mt"
i = m$existd("fero{p1}.mt","ARCFILE")
if i .eq. 1 ARCFILE = "{fero{p1}.mt,ARCFILE}"
!! Output FITS & PS files in VLT compliant name format, i.e.:
!! r.<ARCFILE>.nnnn.fits
!! To set either:
!! In MIDAS
!!   write/key  UseAFDRSFITS/C/1/1 "Y"
!! or in .bashrc
!!   export FEROSUseAFDRSFITS="Y"
!! or in .cshrc or .tcshrc
!!   setenv FEROSUseAFDRSFITS="Y"
i = m$existk("UseAFDRSFITS")
if i .ne. 1 then
  DEFINE/LOCAL UseAFDRSFITS/C/1/1 " " ? +lower_levels
  UseAFDRSFITS = M$SYMBOL("FEROSUseAFDRSFITS")
endif
OFLABEL = "{p7}{p1}-snr"
if "{ARCFILE}" .ne. "fero{p1}.mt" .and. "{UseAFDRSFITS}" .EQ. "Y" OFLABEL = "r.{ARCFILE(1:29)}_snr"

!! Initialize SNR descriptor values
writ/desc {p7}{p1}1.bdf snr/r/1/39 -1. all
writ/desc {p7}{p1}2.bdf snr/r/1/39 -1. all

do iord = {osf(1)} {osf(2)}
  set/format i1
  assi/grap G,{gwids(2)}
  set/format
  mord =  ORDER_LAST - iord + 1
  winsf(1) = midwin({iord}) - winsize/2.
  winsf(2) = winsf(1) + winsize
  set/midas output={p4}
  @@ snr {p1} {winsf(1)},{winsf(2)} 1 ? {P7}
  snr1 = -99.999
  if outputr(4) .gt. 0. snr1 = 1/outputr(4)
  copy/kd snr1/r/1/1 {p7}{p1}1.bdf snr/r/{iord}/1
  @@ snr {p1} {winsf(1)},{winsf(2)} 2 ? {P7}
  snr2 = -99.999
  if outputr(4) .gt. 0. snr2 = 1/outputr(4)
  copy/kd snr2/r/1/1 {p7}{p1}2.bdf snr/r/{iord}/1
  set/midas output=yes
  snrr = snr1/snr2
  set/format i02 f4.1
  write/out "S/N in Order {iord} [m={mord}] [{winsf(1)},{winsf(2)}]: Obj/Sky = {snr1}/{snr2} (ratio = {snrr})"
  set/format
enddo

if "{p5}" .eq. "y" then
  set/format i1
  assi/grap G,{gwids(1)}
  set/format
  set/grap default
  set/grap pmode=1 font=1
  set/grap xax=24,64 yax=0,1000
  plot/axes 24,64,5,1 0,1000,100,50 ? "m-Order" "S/N"
  @@ ferosSetGrapXax 0 40
  label/grap "S/N ratio at blaze wavelengths for {p7}{p1}1 (blue) and {p7}{p1}2 (red)" 0.5,0.975,no ? 0.7 0
  set/grap color=4
  overplot/desc {p7}{p1}1 snr
  set/grap color=2
  overplot/desc {p7}{p1}2 snr
  set/grap default
  if "{p6}" .eq. "y" then
    copy/grap postscript.l
    $ mv postscript.ps {OFLABEL}.ps
  endif
endif
