! "@(#) $Id: ccd_standard_test.prg,v 1.1 2003/01/12 00:32:50 jpritcha Exp $"
! +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
!.IDENTIFICATION: ccd_standard_test.prg    
!.PURPOSE:        Procedure to reduce a standard test
!.AUTHOR:         P. Sinclaire
!.USE:            @@ ccd_standard_test p1 p2 p3 p4 p5 p6
!.                where p1 = number of CCD
!.                      p2 = date of test (format YYYYMMDD)
!.                      p3 = image filename prefix
!.                      p4 = readout mode
!.                      p5 = file number of last image
!.                      p6 = Number of images incl. the 5 bias
!.                      p7 = Print output
!.Algorithm:      Using a set of 5 bias frames, and n pairs of flat
!                 fields the linearity of a CCD is tested. 
!.VERSION:        980706 PSI creation
!.VERSION:      20021022 JDP creation
!      
! +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
DEFINE/PARAMETER P1 ?            C*2  "Number of CCD:"
DEFINE/PARAMETER P2 "00000000"   C*8  "Date of test (YYYYMMDD):"
DEFINE/PARAMETER P3 "test"       C    "Image filename prefix:"
DEFINE/PARAMETER P4 "slow"       C*4  "CCD readout mode:"
DEFINE/PARAMETER P5 1            N    "File number of last image:"
DEFINE/PARAMETER P6 25           N    "Total number of images:"
DEFINE/PARAMETER P7 Y            C*1  "Send to printer (Y/n):"


write/out " "
write/out " "
write/out "   You have just started the linearity reduction package. The"
write/out "   25 images must be in the same order as taken by the CCD"
write/out "   ccd_standardtest test script."
write/out " "
write/out "   This procedure assumes that you run Midas version 95NOV."
write/out " "
write/out "   During execution of this procedure the Midas display window"
write/out "   should be completely inside the area of your workstation,"
write/out "   and not covered by any other display window. Also, do not"
write/out "   inconify this window, or move to another workspace during"
write/out "   execution."
write/out " "
write/out "   All the files created by this procedure will have the suffix"
write/out "   _{p1}_{p2}."
write/out " "
write/out "   At the end of the test the procedure will automatically print"
write/out "   out the test results and images at the assigned printer. A"
write/out "   copy of this test report should be delivered to the ODT team."
write/out " "
write/out "   In case of problems with the reduction package contact Peter"
write/out "   Sinclaire (psinclai@eso.org), or any member of the ODT team."
write/out " "
write/out "                         Good luck!"
write/out " "

!
! Definition of keywords used in this programma
!
define/local dt/d/1/1 0.0
define/local t0/d/1/1 0.0

define/local firstH/i/1/1 0
define/local lastH/i/1/1 0
define/local firstV/i/1/1 0
define/local lastV/i/1/1 0
define/local deltaH/i/1/1 0
define/local deltaV/i/1/1 0
define/local shutLOW/r/1/1 0.0
define/local shutHIGH/r/1/1 0.5
!
! Store frames of the different tests in appropriate image catalogs, define the
! window in which the test are taken and fill the IDENT and O_TIME descriptors
!
! descriptor O_TIME(5): UT time start not filled yet??? OCt96/PS

define/local catal/i/1/1 0
define/local nfrm/i/1/1 0
define/local minmax/r/1/2 0.0 0.0
define/local lastf/i/1/1 0
define/local firsf/i/1/1 0

lastf = {p5}
firsf = {lastf} - {p6} + 1

write/key detec/C/1/20 CCD#{p1}

set/format i4
write/out "    Preparing image catalogs. This may take a while..."
write/out "    processing files: {p3}{firsf}.bdf to {p3}{lastf}.bdf"

set/midas output=no

reset/disp
!     create display first in case not enough color, private Lut is loaded
crea/grap 0 620,480,400,90
crea/disp 0 325,305,10,30
set/midas output=yes

! *** check if the catalogue exists
IF M$EXIST("Std_{p1}_{p2}.cat") .eq. 0 THEN 
   create/icat Std_{p1}_{p2}.cat NULL
   add/icat Std_{p1}_{p2} {p3}*.bdf {p3}{firsf}.bdf,{p3}{lastf}.bdf
   show/icat Std_{p1}_{p2}.cat
   if outputi(1) .ne. {p6} .or. outputi(2) .ne. {p6} then
      WRITE/OUT "*** FATAL error: Number of images does not match."
      Write/out "                 check catalog Std_{p1}_{p2}.cat"
      RETURN/EXIT
   endif
ENDIF

set/midas output=no
 nfrm = 0
LOOP1:
   store/frame in_a Std_{p1}_{p2}.cat 1 CONT1

   copy/dk {in_a} O_TIME/d/7/1 dt 
   copy/dk {in_a} TM_START t0        ! TM_START could double or integer
   t0 = t0 / 3600.0d0
   copy/kd t0/d/1/1 {in_a} O_TIME/d/5/1

   set/format ,f6.1

   write/key tc/c/1/5 {dt}

   nfrm = nfrm + 1

   if nfrm .eq. 1 then

       copy/dk {in_a} START/d/1/1 firstH
       copy/dk {in_a} START/d/2/1 firstV

       copy/dk {in_a} NPIX/i/1/1 deltaH
       copy/dk {in_a} NPIX/i/2/1 deltaV

       lastH = firstH + deltaH - 1
       lastV = firstV + deltaV - 1

       write/descr {in_a} ident/c/1/60 {p1}/BIAS:{tc}
       crea/icat Bias_{p4}_{p1}_{p2} {in_a}

    endif
   if nfrm .gt. 1 .and. nfrm .lt. 6 then
       write/descr {in_a} ident/c/1/60 {p1}/BIAS:{tc}
       add/icat Bias_{p4}_{p1}_{p2} {in_a}
    endif

   if nfrm .eq. 6 then
       write/descr {in_a} ident/c/1/60 {p1}/FF:{tc}
       crea/icat FF_{p4}_{p1}_{p2} {in_a}
    endif
   if nfrm .gt. 6 .and. nfrm .lt. 26 then
       write/descr {in_a} ident/c/1/60 {p1}/FF:{tc}
       add/icat FF_{p4}_{p1}_{p2} {in_a}
    endif

   goto LOOP1

CONT1:

set/format

! Initializing CCD context

clear/context -all
set/con ccdtest

! Clear logfile and set to 60 lines per page
log(7) = 60
log/on
delete/logf
log/off    ! avoid first page output
set/midas output=yes

write/out " "
write/out " "
write/out "     ===================================================="
write/out "                STANDARD TEST REDUCTION PACKAGE"
write/out "     ===================================================="
write/out " "
write/out " "
write/out " "
write/out "                          CCD #{P1}"
write/out " "
write/out " "
write/out "                      DATE TEST {P2}"
write/out " "

log/off

@@ ccdtestb Bias_{p4}_{p1}_{p2}.cat Std_{p4}_{p1}_{p2} median

set/format i1 f6.2
@@ ccdtestt FF_{p4}_{p1}_{p2}.cat Std_{p4}_{p1}_{p2} ? {shutLOW},{shutHIGH}

log/on
log/tof

write/out "     =========================================================="
write/out "     SUMMARY STANDARD TEST CCD#{p1} TAKEN ON {p2} IN {P4} MODE"
write/out "     =========================================================="
write/out " "


write/out "      BIAS"
write/out "      ===="
write/out " "
set/format t8.4
write/out -
     "      5 biases at start test: average = {BIASMEAN} stddev = {BIASSIGM}"
write/out " "
write/out "      There were {hot_pix} hot pixels/columns detected"
write/out " "
write/out " "
write/out "      TRANSFER FUNCTION AND LINEARITY DATA"
write/out "      ===================================="
write/out " "
read/tab Std_{p4}_{p1}_{p2} #3,#5,#6,#7,#8,#9,#11,#12
set/midas output=no
stat/tab Std_{p4}_{p1}_{p2} #5
minmax(1) = outputr(1)
minmax(2) = outputr(2)
stat/tab Std_{p4}_{p1}_{p2} #6
minmax(1) = minmax(1) + outputr(1)
minmax(2) = minmax(2) + outputr(2)
minmax(1) = minmax(1) * ADUCF / 2
minmax(2) = minmax(2) * ADUCF / 2
set/midas output=yes
set/format f6.0

write/out " "
write/out "      Dynamic range explored: from {minmax(1)} to {minmax(2)} e-"
write/out "      Average count rate measured: {counts} ADU/sec."
write/out " "

set/format e12.5

write/out "      CONV. FACTOR AND RON"
write/out "      ===================="
write/out " "
write/out "      C.Fact = {ADUCF} +/- {errcf} e-/ADU"
write/out " "
write/out "      RON    = {ENOISE} +/- {erren} e- RMS"
write/out " "
write/out " "


write/out "      SHUTTER ERROR"
write/out "      ============="
write/out " "
set/format e12.5
write/out -
" Average shutter error derived from fractional count rate = {shuterr} sec"
write/out " "
write/out " "
set/format t7.4

write/out "      LINEARITY ERROR"
write/out "      ==============="
write/out " "
write/out -
" Average linearity error after shutter correct. = +{linerr(6)}/{linerr(5)} %"
write/out " "
write/out " These errors represent maximum deviation in % from the average"
write/out " count rate after correction for shutter delay"

set/format

! Print logfile with results

!print/logf
log/off
!! Save log file
!
$cp $HOME/midwork/FORGR{mid$sess(11:12)}.LOG Std_{p4}_{p1}_{p2}.txt
!
! Create table with data for publication
set/format e12.5 i1
$rm Std_{p4}_{p1}_{p2}.tab
$echo {p2(1:4)} {p2(5:6)} {p2(7:8)} {BIASMEAN} {BIASSIGM} {COUNTS} {ADUCF} {errcf} {ENOISE} {erren} {shuterr} {linerr(6)} {linerr(5)} >Std_{p4}_{p1}_{p2}.tab



log/on
! Make backup

write/out " "
write/out " "
write/out " You just finished the CCD linearity test procedure."
write/out " "
write/out " Please e-mail the file Std_{p4}_{p1}_{p2}.txt to odtls@ls.eso.org"
write/out " "
write/out " A print out of the test results and images is automatically"
write/out " made at the assigned printer. "
write/out " "
write/out " Thank you"
write/out " "

IF "{P7}" .EQ. "Y" THEN

! Print graphics generated by the programme
! In recent versions (e.g.4.13) of a2ps default is to send to printer...
  $ a2ps Std_{p4}_{p1}_{p2}.txt
  $ lpr Std_{p4}_{p1}_{p2}_cbias.ps
  $ lpr Std_{p4}_{p1}_{p2}_aver.ps
  $ lpr Std_{p4}_{p1}_{p2}_hist.ps

  IF M$EXIST("Std_{p4}_{p1}_{p2}_hotpix.ps") .NE. 0  THEN
     $ lpr Std_{p4}_{p1}_{p2}_hotpix.ps
     $ lpr Std_{p4}_{p1}_{p2}_aver_clean.ps
  ENDIF

  $ lpr Std_{p4}_{p1}_{p2}_lin1.ps
  $ lpr Std_{p4}_{p1}_{p2}_lin2.ps
  $ lpr Std_{p4}_{p1}_{p2}_trans.ps
ENDIF

log/on
RETURN
