! "@(#) $Id: initFFsol.prg,v 1.13 2005/04/22 05:22:43 jpritcha Exp $"
! @@ initFFsol.prg
! +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
!
! MIDAS procedure  init.prg	  
! O.Stahl, A.Kaufer	981105
! J.D.Pritchard, 2004-01-29
!
! PURPOSE
! Make an initialisation using a search in the FlatField + SolarSpectrum
! averaged image instead of using the GUESS positions...
!
! USAGE
!
! @@ initFFsol <guess> <action> [threshold=0.8] [y/N] [Order] [Y/n]
!
! with : guess     :  Name of guess session to be used (ThAr<NNNN>)
!        action    :  Action to be taken:
!                       FFsol   :  start from making the new FF+sol mean image
!		   	start   :  full procedure from start
!			flat    :  start with reduction of flat
!			calib	:  start with reduction of calibration	
!        threshold : Search threshold, default value  is 0.8.
!        [Y/N]     : Interactively adjust threshold (default = N)
!        [Order]   : FF to plot
!        [Y/N]     : Re-reduce spectra in sol.cat (default=Y)
![Expected Number of Orders] : When using the search method for order location
!                              specify the number of orders expected to be
!                              found, default 39
!
! +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
!
define/par p1 #           C "Enter name of guess session : "
define/par p2 FFsol       C "Enter action: [FFsol,start/flat/calib] : "
define/par p3 0.80        N "                      Threshold [0.80] : "
define/par p4 N           C "Interactive [y/N] : "
define/par p5 20          N "Order to Plot (20) : "
define/par p6 Y           C "Reprocess sol.cat [Y/n] : "
define/par p7 39          N "Expected Number of Orders (39) : "
!
!
define/local tmpname/c/1/40 " " all
define/local infile/c/1/40 " " all
define/local catal/i/1/1 0 
define/local ind/i/1/1 0 
define/local i/i/1/1 0 
define/local mm/i/1/1 0
define/local action/c/1/40 {p2}
define/local expNO/i/1/1   {p7}
!
define/local iaThresh/c/1/1   "{P4}"
define/local FFname/c/1/10    " "
define/local FFsolname/c/1/10 " "
define/local thresh/r/1/1     {p3}
define/local nthresh/r/1/1    {p3}
define/local p1str/c/1/20     "{p1}"
define/local ymax/r/1/1       0.
define/local iOtoPlot/i/1/1   {p5}
!
set/grap pmode=1 font=1
!
! GAIN and RON now set in separate MIDAS routine ferosSetRONandGAIN.prg
!
! jump according to required action
!
goto {action}

start:
!
! set FEROS context 
! initialize session according to a saved guess session,
! 	i.e., typically the session from the last night
!
! Don't do this otherwise it overrides set/context feroshh
! set/context feros
if "{p1}" .eq. "#" then
 INIT/FEROS
 write/out "Default session parameters loaded"
 p1str = "Default"
else
 INIT/FEROS  {p1}
 write/out "Session parameters loaded from session {p1}"
endif

!
! average flatfield expsoures, 
! averaged flat is used for further reductions
!
@@ average_cat FF.cat

! average calibration exposures, 
! averaged thar is used for wavelength calibration
!
@@ average_cat ThAr.cat

!
FFsol:
if "{action}" .eq. "FFsol" then
! Don't do this otherwise it overrides set/context feroshh
!  set/context feros
  if "{p1}" .eq. "#" then
    INIT/FEROS {WLC_IMG}
    write/out "Session parameters loaded from session {WLC_IMG}"
    p1str = "{WLC_IMG}"
  else
    INIT/FEROS  {p1}
    write/out "Session parameters loaded from session {p1}"
  endif
endif
! Save the name of the true Flat-field image...
FFname = "{RAW_IMG}"
!
! average flatfield + SolarSpectrum expsoures, 
! averaged FFsol is used for tracing orders...
!
! First combine the FF.cat and sol.cat catalogues into FFsol.cat
crea/icat FFsol.cat Null
nextSol:
store/frame infile sol.cat 1 finishSol
  add/icat FFsol.cat {infile}
  goto nextSol
finishSol:
nextFF:
store/frame infile FF.cat 1 finishFF
  add/icat FFsol.cat {infile}
  goto nextFF
finishFF:
!
@@ average_cat FFsol.cat
FFsolname = "{RAW_IMG}"

flat:
! reduce FF: 	order definition
!		background subtraction
!		order straightening
!		order extraction		
!
load/ima {RAW_IMG} sc=-4
!
! set gain acc. to descriptor 
! GAIN and RON now set in separate MIDAS routine ferosSetRONandGAIN.prg
write/out 
@@ ferosSetRONandGAIN {RAW_IMG}
!
! Trace the orders...
searchFFsol:
if thresh .eq. 0.0 then
! Use guess session...
  clear/channel over
  DEFINE/FEROS 
  load/tab {CENTER_TBL} :Fit_pos :Ypos ? 0 1  
else
! Search for the orders...
  searchFF:
  clear/channel overlay
  define/fero ? ? ? 10,25,5 {thresh} SR
  load/tab {CENTER_TBL} :Fit_pos :Ypos ? 0 1
  copy/dk {CENTER_TBL}.tbl ECHORD/I/1/1 NOloc
  if "{iaThresh}" .EQ. "N" .and. NOloc .NE. expNO then
    write/out "{NOloc} NOT {expNO} orders located"
    inquire/key iaThresh "Do you wish to adjust the threshold interactively? (y/N) : " flush
  endif
!
! Interactively adjust Threshold (if chosen in command line options...)
  if "{iaThresh}" .NE. "N" then
    nthresh = thresh
    write/out "Press enter to continue or give a new threshold to search again."
    inquire/key nthresh "Current Threshold is {nthresh} : " flush
    write/out "{nthresh}"
    if nthresh .NE. thresh then
      thresh = nthresh
      goto searchFF
    endif
  endif
! Orders should now be defined... at least 39 of them...
!
! Revert back to the true Flat-field image and carry on with the "normal"
! initialisation...
set/feros RAW_IMG={FFname}
BACKGR/FEROS {RAW_IMG} back
RECTIFY/FEROS back back_str
! prepare cross-order profiles for optimum extraction
EXTRACT/FEROS back_str {RAW_IMG}ext GOYN
! extract flat with standard extraction
EXTRACT/FEROS back_str {RAW_IMG}ext GSNN
SET/FEROS FLAT_IMG={RAW_IMG}ext

clear/channel over

calib:
!reduce ThAr	background subtraction
!		order straightening
!		order extraction		
!		line search
!		wavelength calibration
!
load/ima {WLC_IMG} sc=-4
BACKGR/FEROS {WLC_IMG} back
RECTIFY/FEROS back back_str
EXTRACT/FEROS back_str {WLC_IMG}ext GS

!!! Make 1-Dim FF and ThAr images...
REBIN/FERO {RAW_IMG}ext {LINE_POS_TBL} rebinned I {REBIN_STEP},0.0 ? OS
MERGE/FERO rebinned f{RAW_IMG}

FLAT/FERO {WLC_IMG}ext {FLAT_IMG} f{WLC_IMG}ext
REBIN/FERO f{WLC_IMG}ext {LINE_POS_TBL} rebinned I {REBIN_STEP},0.0 ? OS
MERGE/FERO rebinned f{WLC_IMG}

plots:
SET/FEROS LINE_POS_TBL={WLC_IMG}lines
SEARCH/FERO {WLC_IMG}ext {LINE_W},{LINE_THRES},1 {LINE_POS_TBL}
CALIBRATE/FEROS {LINE_POS_TBL} ? ? ? ? {INIT_WLC}
set/gra xa=20,65 ya=-0.05,0.05 font=1
sele/tab {LINE_POS_TBL}1 abs(:residual).lt.0.05
plot/tab {LINE_POS_TBL}1 :mabs :residual
stat/tab {LINE_POS_TBL}1 :residual
label/graphic "FFsol Search: {p1str}, threshold = {thresh}" 20.5,0.045 0 1 1
label/graphic "rms = {outputr(4)} \AA :: n={outputi(2)}" 20.5,0.040 0 1 1
set/gra default
copy/graph postscript.l
$ mv postscript.ps {FFsolname}-{WLC_IMG}-{p1str}-rms.ps

!! check extracted order profiles in flat image...
stat/imag {RAW_IMG}ext1 [<,{iOtoPlot}:>,{iOtoPlot}]
ymax = outputr(2)
stat/imag {RAW_IMG}ext2 [<,{iOtoPlot}:>,{iOtoPlot}]
if ymax .lt. outputr(2) ymax = outputr(2)
ymax = ymax * 1.05
assi/grap G,1
set/grap yax=0,{ymax}
plot/row {RAW_IMG}ext1 @{iOtoPlot}
assi/grap G,2
set/grap yax=0,{ymax}
plot/row {RAW_IMG}ext2 @{iOtoPlot}

SAVE/FEROS {FFsolname}
write/out "Session parameters saved as session {FFsolname}"

write/out " Making archive..."
@@ autoMkFFsolArchive

if "{P6}" .EQ. "Y" then
  @@ autoreduce_cat sol.cat
endif

exit:
